from text_template import TextTemplate

from rummy.constants.resource_path import TEMPLATE_PATH


class RoundView:
    @staticmethod
    def end_of_round_scores(player):
        return TextTemplate.render(
            TEMPLATE_PATH + '/hand-score.txt',
            player=player.get_name(),
            hand=str(player.hand),
            score=player.hand.get_score()
        )

    @staticmethod
    def this_round_score(round_scores, game_scores):
        return TextTemplate.render(
            TEMPLATE_PATH + '/round-end.txt',
            round_scores=round_scores,
            game_scores=game_scores
        )



from rummy.ui.action import Action
from rummy.ui.action_collection import ActionCollection
from rummy.ui.user_input import UserInput


class MenuActionDialog:

    @staticmethod
    def human_players():
        return ActionCollection(
            Action(range(0, 5), 'Choose number of human players'),
        )

    @staticmethod
    def ai_players(human_players=0):
        return ActionCollection(
            Action(range(
                1 if human_players != 0 else 2,
                5 - int(human_players)
            ), 'Choose number of AI players'),
        )

    @staticmethod
    def next_round():
        return ActionCollection(
            Action('y', 'Ready for the next round'),
        )


if __name__ == '__main__':
    UserInput.create_input(MenuActionDialog.ai_players(UserInput.create_input(MenuActionDialog.human_players())))



# -*- coding: utf-8 -*-

from abc import ABCMeta

from rummy.game.melds import Melds
from rummy.player.hand import Hand


class Player(metaclass=ABCMeta):
    __metaclass__ = ABCMeta

    def __init__(self, num):
        self.num = num
        self.game_score = 0
        self.hand = Hand()
        self.melds = Melds()
        self.round = None
        self.ai_only = False

    def __str__(self):
        return "Player %i" % self.num

    def turn(self, game_round):
        self.round = game_round

    def take_from_deck(self):
        self.hand.draw_card(self.round.deck.take_card())

    def take_from_discard(self):
        self.hand.draw_card(self.round.deck.take_discard())

    def discard(self, user_input):
        user_input = int(user_input) - 1
        self.round.deck.discard_card(self.hand.discard_card(user_input))

    def knock(self):
        self.round.show_knocked = True

    def get_name(self):
        return str(self)

    def update_score(self):
        self.game_score += self.hand.get_score()

    def get_hand(self):
        return self.hand

    def get_game_score(self):
        return self.game_score

    def display_round_score(self):
        return self.hand.score

    def has_someone_knocked(self):
        return self.round.knocked



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
from time import sleep

import colorama

from rummy.controller.ai_controller import AiController
from rummy.controller.human_controller import HumanController
from rummy.game.players import Players
from rummy.game.round import Round
from rummy.game.score import Score
from rummy.player.human import Human
from rummy.ui.menu_action_dialog import MenuActionDialog
from rummy.ui.user_input import UserInput
from rummy.ui.view import View
from rummy.view.round_view import RoundView


class Play:
    def __init__(self):
        print('Welcome to Python Rummy')
        self.colorama()
        players = Players()
        players.choose_players()
        players.choose_opponents()
        self.players = players.get_players()
        self.ai_only = players.is_ai_only()
        self.score = Score(self.players)
        self.round = Round(self.players)
        self.round.deal_cards(self.players)
        self.play_game()

    @staticmethod
    def colorama():
        if 'PYCHARM_HOSTED' in os.environ:
            convert = False  # in PyCharm, we should disable convert
            strip = False
        else:
            convert = None
            strip = None
        colorama.init(convert=convert, strip=strip)

    def play_game(self):
        while self.round.last_turn != len(self.players):
            self.round.prepare_turn()
            player = self.players[self.round.current_player]
            player.turn(self.round)
            # Todo: Views should be agnostic. Each template will have placeholders for data.
            # Todo: Player should return data to be displayed in views placeholders.
            controller = self._select_player_controller(player)
            controller.show_start_turn(player)
            controller.show_knocked(player)
            controller.draw_card(player)
            controller.show_end_turn(player)
            controller.discard_or_knock(player)
            controller.show_discard(player)
            self.round.end_turn()
        View.render(self.end_round())
        sleep(1.2)
        if self.score.is_end_of_game():
            View.render(self.score.show_winners())
        else:
            self.start_new_round()

    def _select_player_controller(self, player):
        if isinstance(player, Human):
            controller = HumanController
        else:
            controller = AiController
        return controller

    def start_new_round(self):
        self.round.rotate_first_player()
        if not self.ai_only:
            self.confirm_start_new_round()
        self.round.prepare_new_round()
        self.round.deal_cards(self.players)
        self.play_game()

    @staticmethod
    def confirm_start_new_round():
        UserInput.create_input(MenuActionDialog.next_round())

    def end_round(self):
        self.score.update_player_scores()
        return RoundView.this_round_score(
            self.score.get_end_of_round_scores(),
            self.score.get_current_game_scores()
        )


# start game
if __name__ == "__main__":
    Play()



# -*- coding: utf-8 -*-

from rummy.player.ai import AI
from rummy.player.human import Human
from rummy.ui.menu_action_dialog import MenuActionDialog
from rummy.ui.user_input import UserInput


class Players:
    def __init__(self):
        self.players = []
        self.number_of_players = -1
        self.number_of_opponents = -1

    def choose_players(self):
        self.number_of_players = int(UserInput.create_input(MenuActionDialog.human_players()))
        for _ in range(int(self.number_of_players)):
            self.players.append(Human(len(self.players) + 1))

    def choose_opponents(self):
        if self.number_of_players in [-1, 0, 1]:
            self.number_of_opponents = int(
                UserInput.create_input(MenuActionDialog.ai_players(self.number_of_players)))
            ai_only = self.is_ai_only()
            for _ in range(int(self.number_of_opponents)):
                self.players.append(AI(len(self.players) + 1, ai_only))

    def is_ai_only(self):
        return True if self.number_of_players == 0 else False

    def get_players(self):
        return self.players



from text_template import TextTemplate

from rummy.constants.resource_path import TEMPLATE_PATH
from rummy.view.player_view import PlayerView


class HumanView(PlayerView):

    @staticmethod
    def turn_start(player):
        return TextTemplate.render(
            TEMPLATE_PATH + '/player-turn-start.txt',
            turn_number=player.round.turn,
            player_number=player.round.current_player + 1,
            score=player.hand.get_score(),
            hand=str(player.hand),
            discard=player.round.deck.show_discard()
        )

    @staticmethod
    def turn_end(player):
        return TextTemplate.render(
            TEMPLATE_PATH + '/player-turn-end.txt',
            hand=str(player.hand),
            key=player.hand.get_key()
        )

    @staticmethod
    def discarded(player):
        return TextTemplate.render(
            TEMPLATE_PATH + '/player-discarded.txt',
            discard=player.round.deck.inspect_discard()
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rummy',
 'rummy.constants',
 'rummy.controller',
 'rummy.deck',
 'rummy.exception',
 'rummy.game',
 'rummy.player',
 'rummy.ui',
 'rummy.view']

package_data = \
{'': ['*'], 'rummy': ['templates/*']}

install_requires = \
['Pygments==2.2.0',
 'ansi-colours==0.2.6',
 'codecov==2.0.15',
 'colorama==0.4.0',
 'coverage==4.5.1',
 'more-itertools==4.3.0',
 'pytest-cov==2.6.0',
 'pytest-mock==1.10.0',
 'pytest==3.10.0',
 'text-template==0.1.4']

entry_points = \
{'console_scripts': ['rummy = rummy.__main__:main']}

setup_kwargs = {
    'name': 'rummy',
    'version': '2.1.5',
    'description': 'Console Rummy game',
    'long_description': None,
    'author': 'Sarcoma',
    'author_email': 'sean@orderandchaoscreative.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.4,<4.0',
}


setup(**setup_kwargs)



from rummy.ui.action import Action
from rummy.ui.action_collection import ActionCollection
from rummy.ui.user_input import UserInput


class PlayerActionDialog:

    @staticmethod
    def pick_up_or_draw():
        return ActionCollection(
            Action('p', 'Pick up discard'),
            Action('d', 'Draw from deck')
        )

    @staticmethod
    def choose_discard():
        return ActionCollection(
            Action(range(1, 9), 'Enter a number to discard a card from your hand')
        )

    @staticmethod
    def choose_discard_or_knock():
        return ActionCollection(
            Action('k', 'You may knock to end the round'),
            Action(range(1, 9), 'Enter a number to discard a card from your hand'),
        )


if __name__ == '__main__':
    UserInput.create_input(PlayerActionDialog.choose_discard())
    UserInput.create_input(PlayerActionDialog.choose_discard_or_knock())



# coding=utf-8

from text_template import TextTemplate

from rummy.constants.resource_path import TEMPLATE_PATH


# Todo: Refactor -- These look more like controllers the current controller is more like a service.


class View:

    @staticmethod
    def render(*args, **kwargs):
        # Just a wrapper/abstraction layer for builtins.print
        # Outputs will go through this class, so that if any
        # complete transformations need to be applied,
        # they can be applied in once place.
        print(*args, **kwargs)

    @staticmethod
    def prepare_template(template, **kwargs):
        complete_template = TEMPLATE_PATH + template
        return TextTemplate.render(complete_template, **kwargs)

